import subprocess
from pathlib import Path
from doka.kernel.definition.entities import UnitEntity
from doka.kernel.repository.repositories import UnitRepository
from doka.kernel.definition.entities import ProjectEntity

base_dir = Path(__file__).resolve().parent


def start(project: ProjectEntity, exclude: list = None):
    NginxConfBuilder(exclude).build(project)
    container_name = 'doka_nginx'
    network = '{}_default'.format(project.name)
    subprocess.run(
        "docker stop {name} && docker rm {name} || true".format(
            name=container_name
        ),
        shell=True,
        check=True
    )
    subprocess.run(
        "docker build -t {image_name} --no-cache {directory} && docker run --name {name} --network {network} -p 80:80 -d {image_name}".format(
            name=container_name,
            image_name='{}_{}'.format(container_name, project.name),
            network=network,
            directory=base_dir
        ),
        shell=True,
        check=True
    )


def stop(project: ProjectEntity):
    container_name = 'doka_nginx'
    subprocess.run(
        "docker stop {name} && docker rm {name} || true".format(
            name=container_name
        ),
        shell=True,
        check=True
    )


def _get_host_name(unit_entity: UnitEntity):
    stack_name, unit_name = unit_entity.name.split(".")
    return "{}.local".format(".".join([unit_name, stack_name, unit_entity.project_name]))


def update_hosts(project: ProjectEntity):
    with open('/etc/hosts') as f:
        content = f.read()

    domains = []
    for unit_entity in UnitRepository(project).get():
        if unit_entity.repo:
            domain = _get_host_name(unit_entity)
            if domain not in content:
                domains.append("127.0.0.1\t{}".format(domain))

    if domains:
        with open('/etc/hosts', "at") as f:
            f.write("\n".join(domains))
            f.write("\n")

    return len(domains)


def _read_template(name):
    return base_dir.joinpath(*["templates", name]).with_suffix(".txt").read_text()


def _write_conf(conf):
    base_dir.joinpath(*["conf", "nginx"]).with_suffix(".conf").write_text(conf)


class NginxConfBuilder:
    def __init__(self, exclude: list = None):
        self._conf_template = _read_template('nginx')
        self._templates = {
            'http_server': _read_template('http_server')
        }
        self._exclude = exclude

    def build(self, project: ProjectEntity):
        data = []
        for unit_entity in UnitRepository(project).get():
            if unit_entity.name not in self._exclude and unit_entity.repo:
                data.append(self._build_http_server_template(unit_entity))

        tpls = {"http_servers": "\n\n".join(data)}
        for tpl, val in tpls.items():
            self._conf_template = self._conf_template.replace("[[{}]]".format(tpl), val)

        _write_conf(self._conf_template)

    def _build_http_server_template(self, unit_entity: UnitEntity):
        vars_map = {
            '$external_host': _get_host_name(unit_entity),
            '$internal_host': "http://{}".format(unit_entity.container_name)
        }
        result = self._templates['http_server']
        for tpl, val in vars_map.items():
            result = result.replace(tpl, val)

        return result


if __name__ == "__main__":
    from doka.kernel.repository.repositories import ProjectRepository

    start(ProjectRepository().get_active())



import fire
from doka.commands import *

__version__ = "0.4.9"


def main():
    fire.Fire({
        'version': __version__,
        'project': Project,
        'unit': Unit,
        "stack": Stack,
        "inspect": Inspect,
        "portainer": Portainer,
        "config": Config,
        "nginx": Nginx
    })



from doka.kernel.repository.repositories import ProjectRepository


class Config:
    def set_active_project(self, project_name):
        repository = ProjectRepository()
        project_entity = repository.get_one(project_name)
        if not project_entity:
            return "Project {} not found".format(project_name)
        repository.set_active(project_entity.name)
        return 'Project {} set as active'.format(project_name)



#!/usr/bin/env python3
import requests

urls = [
    'http://mbagent.ru/kupit-blank-osago-v-moskve/',
    'http://www.evening-kazan.ru/articles/strahovka-po-deshevke-kazancam-prodayut-falshivye-polisy-osago.html',
    'https://blog.goo.ne.jp/pupagifree/e/8ae86ea851556b08229a40ee00a1eb3d?fm=entry_awp_sleep',
    'https://kazanreporter.ru/post/2208_zasada_s_osago',
    'https://lerstrongwind.weebly.com/home/3376263',
    'https://osago-online.tech/osago-optom.html',
    'https://talisman-so.ru/document/formy_dokumientov'
]

for url in urls:
    response = requests.post('http://filter.scaf.icm.local/api/jsonrpc', json={
        "jsonrpc": "2.0",
        "method": "filter.links.avail",
        "id": "4f79ff0e-6cd7-4119-8c88-72ddca5c2d69",
        "params": {
            "url": url
        }
    })

    print(url)
    print(response.json())



from doka.plugins.nginx import nginx
from doka.kernel.repository.repositories import ProjectRepository


class Nginx(object):

    def __init__(self, exclude=None):
        self._project = ProjectRepository().get_active()
        self._exclude = exclude.split(',') or []

    def up(self):
        nginx.start(self._project, self._exclude)

    def restart(self):
        nginx.start(self._project, self._exclude)

    def down(self):
        nginx.stop(self._project)

    def update_hosts(self):
        try:
            count = nginx.update_hosts(self._project)
            return "{} hosts has added".format(count) if count else "No new hosts"
        except PermissionError:
            return "Permission denied. Use sudo to run this command"



from doka.kernel.repository.repositories import ProjectRepository, UnitRepository, StackRepository
from doka.kernel.definition.entities import ProjectSchema, UnitEntity, ProjectEntity
from doka.kernel.services.git import Git
from doka.config import doka_config
from doka.kernel.storage import hooks
from doka.kernel.actions import stack


class Project(object):
    def __init__(self):
        self._projects_repository = ProjectRepository()

    def init(self, name, project_repo, deploy_repo, workdir):
        try:
            project_entity = ProjectSchema().load({
                "name": name,
                "project_repo": project_repo,
                "deploy_repo": deploy_repo,
                "workdir": workdir,
                "metadir": str(doka_config.STORAGE_DIR.joinpath(*["projects", name])),
                "env": "dev"
            }).data
            self._clone_project(project_entity)
            ProjectRepository().save(project_entity)
            return 'Project registered'
        except Exception as e:
            return str(e)

    def remove(self, name):
        try:
            self._projects_repository.remove(name)
            return 'Project removed'
        except Exception as e:
            return str(e)

    def down(self):
        project_entity = self._projects_repository.get_active()
        for stack_entity in StackRepository(project_entity).get():
            stack.down(stack_entity)

    def up(self):
        project_entity = self._projects_repository.get_active()
        for stack_entity in StackRepository(project_entity).get():
            stack.up(stack_entity, project_entity)

    def pull(self):
        project_entity = self._projects_repository.get_active()

        self._clone_project(project_entity)

        hooks.project_post_sync(project_entity.name)
        hooks.move_project_environment_to_stack(project_entity)
        hooks.create_unstacked_stack(project_entity)

        for unit_entity in UnitRepository(project_entity).get_for_build():
            if not unit_entity.codedir.exists():
                Git(unit_entity).clone()
            hooks.move_unit_environment_to_stack(unit_entity)

    def deploy(self, release_branch):
        project_entity = self._projects_repository.get_active()
        unit_entity = UnitEntity(
            name="{}_deploy".format(project_entity.name),
            project_name=None,
            codedir=project_entity.metadir.joinpath("deploy"),
            repo=project_entity.deploy_repo,
            stack_name=None,
            image_name=None
        )
        git = Git(unit_entity)
        if not unit_entity.codedir.exists():
            git.clone()
        else:
            git.fetch(force=False)

        git.create_branch(release_branch)
        git.checkout(release_branch)

        for entry in unit_entity.codedir.iterdir():
            if entry.is_file():
                entry.unlink()
        for stack_entity in StackRepository(project_entity).get():
            if stack_entity.name not in ["dwh", "unstacked"]:
                stack.deploy(project_entity, stack_entity)
        try:
            git.commit('Release: {}'.format(release_branch))
            git.push()
        except Exception as e:
            pass

        git.merge('master')
        git.push()

    def _clone_project(self, project_entity):
        Git(UnitEntity(
            name=project_entity.name,
            project_name=None,
            codedir=project_entity.metadir,
            repo=project_entity.project_repo,
            stack_name=None,
            image_name=None
        )).clone()
